@@ -91,9 +91,11 @@ class MintWelcome():
 
         # Settings button depends on DE
         de_is_cinnamon = False
+        self.theme = None
         if os.getenv("XDG_CURRENT_DESKTOP") in ["Cinnamon", "X-Cinnamon"]:
             builder.get_object("button_settings").connect("clicked", self.launch, "cinnamon-settings")
             de_is_cinnamon = True
+            self.theme = Gio.Settings(schema="org.cinnamon.desktop.interface").get_string("gtk-theme")
         elif os.getenv("XDG_CURRENT_DESKTOP") == "MATE":
             builder.get_object("button_settings").connect("clicked", self.launch, "mate-control-center")
         elif os.getenv("XDG_CURRENT_DESKTOP") == "XFCE":
@@ -161,6 +163,9 @@ class MintWelcome():
 
         scale = window.get_scale_factor()
 
+        self.color = "green"
+        self.dark_mode = False
+
         # Use HIDPI pictures if appropriate
         if scale == 1:
             surface = self.surface_for_path("/usr/share/linuxmint/mintwelcome/legacy.png", scale)
@@ -173,6 +178,15 @@ class MintWelcome():
             surface = self.surface_for_path("/usr/share/linuxmint/mintwelcome/modern-hidpi.png", scale)
             builder.get_object("img_modern").set_from_surface(surface)
 
+        path = "/usr/share/linuxmint/mintwelcome/colors/"
+        if scale == 2:
+            path = "/usr/share/linuxmint/mintwelcome/colors/hidpi/"
+        for color in ["green", "aqua", "blue", "brown", "grey", "orange", "pink", "purple", "red", "sand", "teal"]:
+            builder.get_object("img_" + color).set_from_surface(self.surface_for_path("%s/%s.png" % (path, color), scale))
+            builder.get_object("button_" + color).connect("clicked", self.on_color_button_clicked, color)
+
+        builder.get_object("switch_dark").connect("state-set", self.on_dark_mode_changed)
+
         window.set_default_size(800, 500)
         window.show_all()
 
@@ -231,13 +245,9 @@ class MintWelcome():
         left_icon_size = 0
         center_icon_size = 0
         right_icon_size = 0
-        theme = "Mint-Y"
-        cinnamon_theme = "Mint-Y-Dark"
         if style == LAYOUT_STYLE_LEGACY:
             applets = applets_legacy
             panel_size = 27
-            theme = "Mint-X"
-            cinnamon_theme = "Linux Mint"
             menu_label = "Menu"
         elif style == LAYOUT_STYLE_NEW:
             panel_size = 40
@@ -248,13 +258,43 @@ class MintWelcome():
         settings.set_strv("enabled-applets", applets)
         settings.set_string("app-menu-label", menu_label)
         settings.set_string("panel-zone-icon-sizes", "[{\"panelId\": 1, \"left\": %s, \"center\": %s, \"right\": %s}]" % (left_icon_size, center_icon_size, right_icon_size))
-
-        Gio.Settings('org.cinnamon.desktop.wm.preferences').set_string('theme', theme)
-        Gio.Settings('org.cinnamon.theme').set_string('name', cinnamon_theme)
-        Gio.Settings('org.cinnamon.desktop.interface').set_string('icon-theme', theme)
-        Gio.Settings('org.cinnamon.desktop.interface').set_string('gtk-theme', theme)
         os.system("cinnamon --replace &")
 
+    def on_dark_mode_changed(self, button, state):
+        self.dark_mode = state
+        self.change_color()
+
+    def on_color_button_clicked(self, button, color):
+        self.color = color
+        self.change_color()
+
+    def change_color(self):
+        theme = "Mint-Y"
+        wm_theme = "Mint-Y"
+        cinnamon_theme = "Mint-Y-Dark"
+        if self.dark_mode:
+            theme = "%s-Dark" % theme
+            wm_theme = "Mint-Y-Dark"
+        if self.color != "green":
+            theme = "%s-%s" % (theme, self.color.title())
+            cinnamon_theme = "Mint-Y-Dark-%s" % self.color.title()
+
+        if os.getenv("XDG_CURRENT_DESKTOP") in ["Cinnamon", "X-Cinnamon"]:
+            settings = Gio.Settings(schema="org.cinnamon.desktop.interface")
+            settings.set_string("gtk-theme", theme)
+            settings.set_string("icon-theme", theme)
+            Gio.Settings(schema="org.cinnamon.desktop.wm.preferences").set_string("theme", wm_theme)
+            Gio.Settings(schema="org.cinnamon.theme").set_string("name", cinnamon_theme)
+        elif os.getenv("XDG_CURRENT_DESKTOP") == "MATE":
+            settings = Gio.Settings(schema="org.mate.interface")
+            settings.set_string("gtk-theme", theme)
+            settings.set_string("icon-theme", theme)
+            Gio.Settings(schema="org.mate.Marco.general").set_string("theme", wm_theme)
+        elif os.getenv("XDG_CURRENT_DESKTOP") == "XFCE":
+            subprocess.call(["xfconf-query", "-c", "xsettings", "-p", "/Net/ThemeName", "-s", theme])
+            subprocess.call(["xfconf-query", "-c", "xsettings", "-p", "/Net/IconThemeName", "-s", theme])
+            subprocess.call(["xfconf-query", "-c", "xfwm4", "-p", "/general/theme", "-s", theme])
+
     def visit(self, button, url):
         subprocess.Popen(["xdg-open", url])
 
